local ck = chaosKart

------------------
-- RACE EFFECTS --
------------------
local kartItemMobjs = {MT_BANANA, MT_ORBINAUT, MT_JAWZ, MT_JAWZ_DUD, MT_SSMINE, MT_SSMINE_SHIELD, MT_BALLHOG, MT_EGGMANITEM}

ck.raceeffects[1] = {
	name = "Icy Floor",
	id = "IcyFloorPerm",
	desc = "The floor became icy",
	startsound = sfx_ideya,
	synchnewjoiners = true,
	startfunc = function(resynch)
		for sector in sectors.iterate
			if (sector.floorpic ~= "F_SKY1" or sector.floorpic ~= "PIT")
				sector.floorpic = "ICEFLRA"
			end

			for rover in sector.ffloors()
				if rover.valid and (rover.toppic ~= "F_SKY1" or rover.toppic ~= "PIT")
					rover.toppic = "ICEFLRA"
				end
			end
		end
		P_SwitchWeather(2)
	end,
	mobjthinker = function(mo)
		for k = 1, #kartItemMobjs do
			if mo.type == kartItemMobjs[k] and mo.type ~= MT_BANANA and mo.type ~= MT_ORBINAUT and mo.type ~= MT_JAWZ and mo.type ~= MT_BALLHOG then
				mo.friction = FRACUNIT
				P_Thrust(mo, mo.angle, 35*FRACUNIT)
				break
			end
		end
	end,
	thinker = function()		
		for p in players.iterate
			if p.mo and p.mo.valid then p.mo.friction = FRACUNIT end
		end
	end,
}

ck.raceeffects[2] = {
	name = "Hail",
	id = "HailPerm",
	desc = "It's raining ice!",
	startsound = sfx_ideya,
	synchnewjoiners = true,
	startfunc = function(resynch)
		P_SwitchWeather(2)
	end,
	thinker = function()		
		local possiblePlayers = {}
		for p in players.iterate
			if p.mo and p.mo.valid and not p.spectator then possiblePlayers[#possiblePlayers+1] = p.mo end
		end

		if #possiblePlayers and not (leveltime%15) then
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local ice = P_SpawnMobj(tmo.x+P_RandomRange(-1500, 1500)<<FRACBITS, tmo.y+P_RandomRange(-1500, 1500)<<FRACBITS, tmo.ceilingz-P_RandomRange(0, 150)<<FRACBITS, MT_FIREBALL)
			ice.fuse = 25*TICRATE
			ice.frozen = true
		end
	end,
}

ck.raceeffects[3] = {
	name = "Windy",
	id = "WindyPerm",
	desc = "Super windy!",
	startsound = sfx_ideya,
	startfunc = function(resynch)
		server.windang2 = P_RandomRange(0, 359)*ANG1
	end,
	playerthink = function(p)
		p.mo.momx = $ - cos(server.windang)/(P_IsObjectOnGround(p.mo) and 4 or 2)
		p.mo.momy = $ - sin(server.windang)/(P_IsObjectOnGround(p.mo) and 4 or 2)
	end,
}

ck.raceeffects[4] = {
	name = "Earthquake",
	id = "Earthquake",
	desc = "The ground is shaking!",
	startsound = sfx_ideya,
	thinker = function()
		if not replayplayback then P_StartQuake(7*FRACUNIT, TICRATE) end
	end,
	mobjthinker = function(mo)
		for k = 1, #kartItemMobjs do
			if P_IsObjectOnGround(mo) and not mo.player and mo.type == kartItemMobjs[k] then
				P_SetObjectMomZ(mo, P_RandomRange(10, 20)*FRACUNIT)
				P_Thrust(mo, P_RandomRange(0, 360)*ANG1, P_RandomRange(1, 10)*FRACUNIT)
			end
		end
	end,
}

ck.raceeffects[5] = {
	name = "High Speed Racing",
	id = "HighSpeed",
	desc = "High Speed Racing!",
	startsound = sfx_ideya,
	playerthink = function(p)
		if p.kartstuff[k_offroad] == 0 then p.kartstuff[k_boostpower] = FRACUNIT + FRACUNIT/3 end
	end,
}

ck.raceeffects[6] = {
	name = "200CC Racing",
	id = "HigherSpeed",
	desc = "200CC Racing!",
	startsound = sfx_ideya,
	playerthink = function(p)
		if p.kartstuff[k_offroad] == 0 then p.kartstuff[k_boostpower] = 2*FRACUNIT end
	end,
}

ck.raceeffects[7] = {
	name = "Weak Exhaust",
	id = "WeakExhaust",
	desc = "A weaker exhaust! I guess this one'll be slower.",
	startsound = sfx_ideya,
	playerthink = function(p)
		if p.kartstuff[k_offroad] == 0 then p.kartstuff[k_boostpower] = FRACUNIT - FRACUNIT/3  end
	end,
}

ck.raceeffects[8] = {
	name = "Sunlight",
	id = "Sunlight",
	desc = "Items burn, setting you on fire.",
	startsound = sfx_ideya,
	startfunc = function(resynch)
		for sector in sectors.iterate do
			sector.lightlevel = 255
		end
	end,
	playerthink = function(p)
		if p.mo.onfire then
			p.mo.onfire = $-1 
			if p.kartstuff[k_offroad] == 0 then p.kartstuff[k_boostpower] = 3*FRACUNIT/4 end

			p.mo.colorized = true
			p.mo.color = SKINCOLOR_ORANGE
			local smoke = P_SpawnMobj(p.mo.x+P_RandomRange(-25, 25)*p.mo.scale, p.mo.y+P_RandomRange(-25, 25)*p.mo.scale, p.mo.z+P_RandomRange(5, 30)*p.mo.scale, MT_SMOKE)
			if not S_SoundPlaying(p.mo, sfx_kc51) then S_StartSound(p.mo, sfx_kc51) end

			if p.mo.onfire <= 0 then
				p.mo.colorized = false
				p.mo.color = p.skincolor
				p.mo.onfire = nil
			end
		end
	end,
	mobjthinker = function(mo)
		for k = 1, #kartItemMobjs do
			if mo.type == kartItemMobjs[k] then
				mo.color = SKINCOLOR_ORANGE
				mo.colorized = true
				local smoke = P_SpawnMobj(mo.x+P_RandomRange(-25, 25)*mo.scale, mo.y+P_RandomRange(-25, 25)*mo.scale, mo.z+P_RandomRange(5, 30)*mo.scale, MT_SMOKE)
				if not S_SoundPlaying(mo, sfx_kc51) then S_StartSound(mo, sfx_kc51) end
				break
			end
		end
	end,
	spineffect = function(p, i, s)
		for k = 1, #kartItemMobjs do
			if i.type == kartItemMobjs[k] then
				p.kartstuff[k_spinouttimer] = $*2
				p.mo.onfire = 10*TICRATE
				break
			end
		end
	end,
	explodeeffect = function(p, i, s)
		for k = 1, #kartItemMobjs do
			if i.type == kartItemMobjs[k] then
				p.kartstuff[k_spinouttimer] = $*2
				p.mo.onfire = 10*TICRATE
				break
			end
		end
	end,
}

-----------------------
-- RARE RACE EFFECTS --
-----------------------

ck.raceeffects[9] = {
	name = "Banana Rain",
	id = "BananaRain",
	desc = "Oooo... Banana...",
	soundplay = sfx_obanan,
	rare = true,
	thinker = function()
		local possiblePlayers = {}
		for p in players.iterate
			if p.mo and p.mo.valid and not p.spectator then possiblePlayers[#possiblePlayers+1] = p.mo end
		end

		if #possiblePlayers and not (leveltime%5) then
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local banana = P_SpawnMobj(tmo.x+P_RandomRange(-2500, 2500)<<FRACBITS, tmo.y+P_RandomRange(-2500, 2500)<<FRACBITS, tmo.ceilingz-P_RandomRange(0, 150)<<FRACBITS, MT_BANANA)
			banana.fuse = 30*TICRATE
		end
	end
}

ck.raceeffects[10] = {
	name = "Bonkless",
	id = "Bonkless",
	desc = "Can you make a bonkless run? I hope so, or else.",
	soundplay = sfx_athun1,
	rare = true,
	playerthink = function(p)
		if p.mo and p.mo.valid and not p.spectator then
			if (p.mo.eflags & MFE_JUSTBOUNCEDWALL or p.kartstuff[k_justbumped] >= 1) and p.kartstuff[k_growshrinktimer] <= 1 and p.playerstate == PST_LIVE and not p.exiting and not p.kartstuff[k_invincibilitytimer] then
				K_SpawnMineExplosion(p.mo, p.mo.color)
				S_StartSound(p.mo, sfx_s3k4e)
				P_KillMobj(p.mo)
			end
		end
	end
}

ck.raceeffects[11] = {
	name = "Meteor Shower",
	id = "MeteorShower",
	desc = "Are those comets? Make a wish!",
	soundplay = sfx_athun1,
	rare = true,
	mobjthinker = function(mo) -- hack
		if mo.type == MT_FALLINGROCK and mo.meteor and P_IsObjectOnGround(mo) then
			K_SpawnMineExplosion(mo, SKINCOLOR_CYAN)

			local explode = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPBEXPLOSION)
			explode.scale = FRACUNIT+(FRACUNIT/2)

			P_StartQuake(7*FRACUNIT, 10)
			P_RemoveMobj(mo)
			return
		end
	end,
	thinker = function()
		local possiblePlayers = {}
		for p in players.iterate
			if p.mo and p.mo.valid and not p.spectator then possiblePlayers[#possiblePlayers+1] = p.mo end
		end

		if #possiblePlayers and not (leveltime%20) then
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local rock = P_SpawnMobj(tmo.x+P_RandomRange(-1250, 1250)<<FRACBITS, tmo.y+P_RandomRange(-1250, 1250)<<FRACBITS, tmo.ceilingz-P_RandomRange(0, 150)<<FRACBITS, MT_FALLINGROCK)
			rock.scale = P_RandomRange(1, 3)*P_RandomFixed()
			rock.fuse = 20*TICRATE
			rock.meteor = true
			rock.color = SKINCOLOR_RED
			rock.colorized = true
		end
	end
}

ck.raceeffects[12] = {
	name = "Invisibility",
	id = "Invisibility",
	desc = "Everything is invisible!",
	soundplay = sfx_athun1,
	rare = true,
	mobjthinker = function(mo)
		if mo and mo.valid then mo.flags2 = $|MF2_DONTDRAW end
	end
}