/*
	Speccy Tracks Remake!

	Freezing by Spectra
	- Adapted from my first lua for kart euphorica.
	- Mostly Copypasted from my Chaoskart mod.
*/

freeslot("SPR_BUTN", "SPR_ICE_")
freeslot("sfx_hwooih", "sfx_icehit", "sfx_freeze")
freeslot("S_ICEBALL", "S_MASH")
freeslot("MT_MASH")

states[S_ICEBALL] = {SPR_ICE_, A, 50, nil, 0, 0, S_ICEBALL}
states[S_MASH] = {SPR_BUTN, FF_FULLBRIGHT|FF_ANIMATE|A, 1, nil, 0, 0, S_NULL}

mobjinfo[MT_MASH] = {
    spawnstate = S_MASH,
    radius = 1*FRACUNIT,
    height = 1*FRACUNIT,
    flags = MF_SCENERY|MF_NOGRAVITY|MF_FLOAT|MF_NOCLIP|MF_NOCLIPHEIGHT
}

addHook("MobjThinker", function(mo)
	if not mo or not mo.valid then return end

	if mo.frozen then
		mo.color = SKINCOLOR_CYAN
		mo.colorized = true
		mo.flags = $|MF_GRENADEBOUNCE|MF_BOUNCE|MF_PAIN

		mo.sprite = SPR_ICEB
		mo.frame = FF_FULLBRIGHT|E
	else
		mo.sprite = SPR_FBLL
		mo.frame = 7+(leveltime%16)

		mo.scale = 4*mapobjectscale/5
		mo.flags = $|MF_GRENADEBOUNCE|MF_BOUNCE|MF_PAIN

		local ghost = P_SpawnGhostMobj(mo)
		ghost.destscale = 1

		local steam = P_SpawnMobj(mo.x+P_RandomRange(-45, 45)*FRACUNIT, mo.y+P_RandomRange(-45, 45)*FRACUNIT, mo.z+P_RandomRange(0, 30)*FRACUNIT, MT_SMOKE)
		P_SetObjectMomZ(steam, P_RandomRange(1, 3)*FRACUNIT)
	end

	if P_IsObjectOnGround(mo) then
		mo.momz = 2*FRACUNIT*P_MobjFlip(mo)
		if mo.frozen then S_StartSound(mo, sfx_icehit) end
	end
end, MT_FIREBALL)

local function shouldHurt(p)
    return not (p.powers[pw_flashing] > 0 or p.kartstuff[k_squishedtimer] > 0 or p.kartstuff[k_spinouttimer] > 0 or p.kartstuff[k_invincibilitytimer] > 0 or p.kartstuff[k_growshrinktimer] > 0 or p.kartstuff[k_hyudorotimer] > 0 or (HugeQuest and p.hugequest.huge > 0) or (G_BattleGametype() and ((p.kartstuff[k_bumper] <= 0 and p.kartstuff[k_comebacktimer]) or p.kartstuff[k_comebackmode] == 1)))
end

addHook("ShouldDamage", function(mo, inf, src, dmg, dmgtype)
	if not mo or not mo.valid or not inf or not inf.valid then return end
	if not mo.player then return end
	local p = mo.player

	-- Fire Flower sender immunity.
	if (inf.type == MT_FIREBALL or inf.type == MT_SPINFIRE) and (inf.flags & MF_PAIN) then
		if mo == inf.source then
			return false
		else
			if inf.frozen then
				S_StartSound(mo, sfx_freeze)
				if shouldHurt(p) then
					p.kartstuff[k_wipeoutslow] = 5*TICRATE
					mo.frozen = 85
				else
					K_DoInstashield(p)
					return false
				end
			end

			P_RemoveMobj(inf)
			return
		end
	end
end, MT_PLAYER)

// Thinkframe
addHook("ThinkFrame", function()
	for p in players.iterate do
		if p.mo and p.mo.valid then
			if p.mo.frozen then
				p.freezeangle = $ or p.mo.angle

				p.mo.color = SKINCOLOR_JAWZ
				p.mo.colorized = true
				p.mo.angle = p.freezeangle
				p.mo.friction = FRACUNIT

				p.kartstuff[k_spinouttimer] = 2
				p.kartstuff[k_wipeoutslow] = 0
				p.kartstuff[k_offroad] = 0
				p.kartstuff[k_boostpower] = min($, FRACUNIT/6)

				p.mo.frozen = $-1

				local mash = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z + P_GetPlayerHeight(p), MT_MASH)
				mash.frame = (leveltime%10)/3
				mash.tics = 1

				if p.buttons.drift == 1 or p.buttons.accelerate == 1 or p.buttons.attack == 1 or p.buttons.brake == 1 then
					S_StartSound(p.mo, sfx_s3k94)
					p.mo.frozen = $-10

					local debris = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SMK_ICEBLOCK_DEBRIS)
					debris.angle = FixedAngle(P_RandomRange(0,360)<<FRACBITS)
					P_InstaThrust(debris, debris.angle, P_RandomRange(3,18)*(FRACUNIT/4))
					debris.momz = P_RandomRange(4,8)<<FRACBITS
				end

				if p.mo.frozen <= 0 then
					S_StartSound(p.mo, sfx_s3k82)
					p.mo.angle = $
					p.mo.colorized = false
					p.mo.color = p.skincolor
					p.mo.frozen = nil
					p.freezeangle = nil

					local debris
					for i = 1, 20 do
						debris = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_SMK_ICEBLOCK_DEBRIS)
						debris.angle = FixedAngle(P_RandomRange(0,360)<<FRACBITS)
						P_InstaThrust(debris, debris.angle, P_RandomRange(3,18)*(FRACUNIT/4))
						debris.momz = P_RandomRange(4,8)<<FRACBITS
					end
				end
			end
		end
	end
end)