local ck = chaosKart

local function setupNextEvent()
	local eventTable = {}

	for i = 1, #ck.events do
		if ck.events[i].deathroulette then eventTable[#eventTable+1] = i end
	end

	local id = P_RandomRange(1, #eventTable)
	server.nextDeathEvent = eventTable[id]
end

ck.eventTimer = function()
	if not server or not server.valid then return end

	// Get Player Numbers
	server.playercount = 0
	for p in players.iterate
		if p.mo and p.mo.valid then server.playercount = $+1 end
	end

	// Continuous Events
	if server.eventvars then
		for k in pairs(server.eventvars) do
			if server.eventvars[k] and server.eventvars[k] > 0 and ck.events[k] then
				server.eventvars[k] = $-1
				if ck.events[k].thinker then ck.events[k].thinker() end

				if ck.events[k].playerthink then
					for p in players.iterate do
						if p.mo and p.mo.valid then ck.events[k].playerthink(p) end
					end
				end

				if server.eventvars[k] <= 0 and ck.events[k].onfinish then
					ck.events[k].onfinish()
				end
			end
		end
	end

	if server.deathRoulette then
		// Introduce the Death Roulette
		if server.deathRouletteIntro then
			server.chaoskarttimer = 5*TICRATE
			server.deathRouletteIntro = $-1
			if server.deathRouletteIntro <= 0 then
				server.deathRouletteIntro = nil
				setupNextEvent()
			end

			return
		end

		// Slowly count down to impending doom.
		server.chaoskarttimer = $-1
		if server.chaoskarttimer == 2*TICRATE then S_StartSound(nil, sfx_ctimer) end

		if server.chaoskarttimer <= 0 then
			S_StartSound(nil, sfx_kc48)

			local eventData = ck.events[server.nextDeathEvent]

			if eventData.runfunc then eventData.runfunc(5*TICRATE) end
			server.curEvent = server.nextDeathEvent

			if eventData.varlength then
				if not server.eventvars then server.eventvars = {} end
				server.eventvars[server.nextDeathEvent] = 5*TICRATE
			end

			server.chaoskarttimer = 5*TICRATE
			setupNextEvent()
		end

		if not (leveltime%455) then P_StartQuake(15*FRACUNIT, 11) end

		return
	end

	// Delay before next countdown.
	if server.timerDelay then
		server.timerDelay = $-1
		if server.timerDelay <= 0 then server.timerDelay = nil end
		return
	end

	// Timer.
	if not server.chaoskarttimer then server.chaoskarttimer = cv_starteventtime.value*TICRATE end
	if (leveltime > 7*TICRATE) then server.chaoskarttimer = $-1 end

	if server.chaoskarttimer <= 0 then
		if P_RandomChance(cv_deathroulettechance.value*FRACUNIT/100) then
			ck.doDeathRoulette()
		else
			ck.runRandEvent()
			server.chaoskarttimer = cv_starteventtime.value*TICRATE
		end
	end
end

ck.runRaceEffect = function()
	if not server.curRaceEffect or not ck.raceeffects[server.curRaceEffect] then return end
	local effect = ck.raceeffects[server.curRaceEffect]

	if (leveltime > 6*TICRATE or effect.ignorestarttime) then
		if effect.thinker then effect.thinker() end

		if effect.playerthink then
			for p in players.iterate do
				effect.playerthink(p)
			end
		end
	end
end

ck.runMobjThinker = function(mo)
	if not mo or not mo.valid or (mo.flags & MF_NOTHINK) then return end
	if not server.curRaceEffect or not ck.raceeffects[server.curRaceEffect] then return end
	local effect = ck.raceeffects[server.curRaceEffect]

	if (leveltime > 6*TICRATE or effect.ignorestarttime) then
		if effect.mobjthinker then
			return effect.mobjthinker(mo)
		end
	end
end